/*
 *    GS2MGPPRetrieve.java
 *    Copyright (C) 2002 New Zealand Digital Library, http://www.nzdl.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.greenstone.gsdl3.service;

// greenstone classes
import org.greenstone.mgpp.*;
import org.greenstone.gdbm.*;
import org.greenstone.gsdl3.util.*;

// xml classes
import org.w3c.dom.Document; 
import org.w3c.dom.Node; 
import org.w3c.dom.Text; 
import org.w3c.dom.Element; 
import org.w3c.dom.NodeList; 

// general java classes
import java.util.HashMap;
import java.util.Vector;
import java.util.Set;
import java.util.Map;
import java.util.Iterator;
import java.util.Locale;

/**
 * A ServiceRack class for retrieval in greenstone 2 MGPP collections
 *
 * @author <a href="mailto:kjdon@cs.waikato.ac.nz">Katherine Don</a>
 * @version $Revision: 3678 $
 * @see ServiceRack
 */
public class GS2MGPPRetrieve 
    extends ServiceRack {

    // these strings must match what is found in the properties file
    // the services on offer
    private static final String DOCUMENT_RETRIEVE_SERVICE = "DocumentRetrieve";
    private static final String METADATA_RETRIEVE_SERVICE = "MetadataRetrieve";

    // params used
    private static final String LEVEL_PARAM = "level";

    // elements used in the config file that are specific to this class
    private static final String DEFAULT_LEVEL_ELEM = "defaultLevel";
    
    private MGPPWrapper mgpp_src_=null;
    private GDBMWrapper gdbm_src_=null;
    
    private String default_level_=null;

    private Element config_info_ = null;

    public GS2MGPPRetrieve() {
	mgpp_src_ = new MGPPWrapper();
	gdbm_src_ = new GDBMWrapper();

    }
    
    /** configure this service */
    public boolean configure(Element info) {

	System.out.println("configuring GS2MGPPRetrieve");
	config_info_ = info;

	// get the default level out of <defaultLevel>
	Element def = (Element)GSXML.getChildByTagName(info, DEFAULT_LEVEL_ELEM);
	if (def !=null) {
	    default_level_ = def.getAttribute(GSXML.NAME_ATT);
	}
	if (default_level_==null||default_level_.equals("")) {
	    System.err.println("Error: default level not specified!");
	    return false;
	}

	// set up which services are available for this collection
	Element e = null;
	// these entries should reflect the build config file - some services may not be available depending on how the collection was built.

	// set up short_service_info_ - for now just has name and type
	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_RETRIEVE);
	e.setAttribute(GSXML.NAME_ATT, DOCUMENT_RETRIEVE_SERVICE);
	short_service_info_.appendChild(e);

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_RETRIEVE);
	e.setAttribute(GSXML.NAME_ATT, METADATA_RETRIEVE_SERVICE);
	short_service_info_.appendChild(e);

	// set up service_info_map_ - for now, just has the same elements as above
	// should have full details about each service incl params lists etc.
	// do the text query one - for now a static list. later use buildcfg.xml values to dynamically change this
	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_RETRIEVE);
	e.setAttribute(GSXML.NAME_ATT, DOCUMENT_RETRIEVE_SERVICE);
	service_info_map_.put(DOCUMENT_RETRIEVE_SERVICE, e);
	
	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_RETRIEVE);
	e.setAttribute(GSXML.NAME_ATT, METADATA_RETRIEVE_SERVICE);
	service_info_map_.put(METADATA_RETRIEVE_SERVICE, e);

	if (gdbm_src_.openDatabase(GSFile.GDBMDatabaseFile(site_home_, cluster_name_), GDBMWrapper.READER)) {
	    return true;
	}
	else {
	    System.err.println("couldn't open gdbm database!");
	    return false;
	}
    }
  

    /** creates a display element containing all the text strings needed to display the service page, in the language specified 
     * these retrieval services dont get displayed to the users - they are only used internally by the actions. so this returns an empty display element*/
    protected Element createServiceDisplay(String service, String lang) {
	return doc_.createElement(GSXML.DISPLAY_ELEM);
    }


    /** retrieve a document */
    protected Element processDocumentRetrieve(Element request) {

	// where the mgpp text files are
	String basedir = GSFile.collectionBaseDir(site_home_, 
						  cluster_name_);
	String textdir = GSFile.collectionTextPath(cluster_name_);

	// an empty result
	Element result = doc_.createElement(GSXML.RESPONSE_ELEM);
	String from = GSPath.appendLink(cluster_name_, DOCUMENT_RETRIEVE_SERVICE);
	result.setAttribute(GSXML.FROM_ATT, from);
	result.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_QUERY);
	Element result_doc = doc_.createElement(GSXML.CONTENT_ELEM);
	result.appendChild(result_doc);
	
	// get param list and content 
	Element param_elem=null;
	Element content_elem=null;
	Node n = request.getFirstChild();
	while (n!=null) {
	  String node_name = n.getNodeName();
	  if (node_name.equals(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER)) {
	      param_elem = (Element)n;
	  } else if (node_name.equals(GSXML.CONTENT_ELEM)) {
	      content_elem = (Element)n;
	  }
	  n = n.getNextSibling();
	}

	if (param_elem==null || content_elem==null) {
	    System.err.println("GS2MGPPRetrieve: malformed request sent to DocumentRetrieve service");
	    return result; // empty result
	}
	
	HashMap params = GSXML.extractParams(param_elem);

	// get docs from mgpp
	String level = (String)params.get(LEVEL_PARAM); // level at which to retrieve a doc
	if (level==null) {
	    level=default_level_;
	}
	// get the text for each doc in the list
	String []ids = GSXML.getDocumentNameList(content_elem);
	for (int j=0; j<ids.length; j++) {
	    long real_num = gdbm_src_.oid2Docnum(ids[j]);
	    String document = mgpp_src_.getDocument(basedir, textdir,  level, real_num);
	    // for now, stick it in a text node - eventually should be parsed as xml??
	    // something funny with the doc - 
	    Element new_doc = GSXML.createDocumentElement(doc_, ids[j]);
	    GSXML.addDocText(doc_, new_doc, document);
	    result_doc.appendChild(new_doc);
	}
	return result;
    }

    /** retrieve metadata */
    protected Element processMetadataRetrieve(Element request) {
	Element result = doc_.createElement(GSXML.RESPONSE_ELEM);
	String from = GSPath.appendLink(cluster_name_, METADATA_RETRIEVE_SERVICE);
	result.setAttribute(GSXML.FROM_ATT, from);
	result.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_QUERY);
	Element result_content = doc_.createElement(GSXML.CONTENT_ELEM);
	result.appendChild(result_content);
	Element document_list = doc_.createElement(GSXML.DOCUMENT_ELEM+GSXML.LIST_MODIFIER);
	result_content.appendChild(document_list);
	// get the metadata
	
	Element content = (Element)request.getElementsByTagName(GSXML.CONTENT_ELEM).item(0);
	if (content==null) {
	    // error: query had no content!! - should make an error message
	    return result;
	}
	String []metas = GSXML.getMetaNameList(content);
	String []ids = GSXML.getDocumentNameList(content);
	for (int j=0; j<ids.length; j++) { // for each document
	    Element doc = GSXML.createDocumentElement(doc_, ids[j]);
	    Element list = GSXML.addMetaList(doc_, doc);
	    DBInfo info = gdbm_src_.getInfo(ids[j]);
	    for (int m=0; m<metas.length; m++) {
		String value = info.getInfo(metas[m]);
		GSXML.addMetadata(doc_, list, metas[m], value);
	    }
	    document_list.appendChild(doc);
	}
	return result;
    }




}
 

